from __future__ import annotations

from aamva_parser.field_mapping import FieldMapper


def _base_fields() -> dict[str, str]:
    return FieldMapper().copy_fields()


class VersionOneFieldMapper:
    def __init__(self) -> None:
        self.fields = _base_fields()
        self.fields["drivers_license_id"] = "DBJ"
        self.fields["last_name"] = "DAB"
        self.fields["driver_license_name"] = "DAA"

    def field_for(self, key: str) -> str:
        return self.fields.get(key) or key


class VersionTwoFieldMapper:
    def __init__(self) -> None:
        self.fields = _base_fields()
        self.fields["first_name"] = "DCT"

    def field_for(self, key: str) -> str:
        return self.fields.get(key) or key


class VersionThreeFieldMapper(VersionTwoFieldMapper):
    pass


class VersionFourFieldMapper:
    def __init__(self) -> None:
        self.fields = _base_fields()

    def field_for(self, key: str) -> str:
        return self.fields.get(key) or key


class VersionFiveFieldMapper(VersionFourFieldMapper):
    pass


class VersionSixFieldMapper(VersionFourFieldMapper):
    pass


class VersionSevenFieldMapper(VersionFourFieldMapper):
    pass


class VersionEightFieldMapper(VersionFourFieldMapper):
    pass


class VersionNineFieldMapper(VersionFourFieldMapper):
    pass


class VersionTenFieldMapper(VersionFourFieldMapper):
    pass


class VersionElevenFieldMapper(VersionFourFieldMapper):
    pass


class VersionTwelveFieldMapper:
    def __init__(self) -> None:
        self.fields = {
            **_base_fields(),
            "cdl_indicator": "DDM",
            "non_domiciled_indicator": "DDN",
            "enhanced_credential_indicator": "DDO",
            "permit_indicator": "DDP",
        }
        self.fields.pop("last_name_alias", None)
        self.fields.pop("first_name_alias", None)
        self.fields.pop("suffix_alias", None)

    def field_for(self, key: str) -> str:
        return self.fields.get(key) or key



from __future__ import annotations

from datetime import datetime
from re import escape as re_escape

from aamva_parser.enums import (
    EyeColor,
    Gender,
    HairColor,
    IssuingCountry,
    NameSuffix,
    Truncation,
)
from aamva_parser.field_mapping import FieldMapper, FieldMapping
from aamva_parser.regex_util import Regex


class FieldParser:
    INCHES_PER_CENTIMETER: float = 0.393701

    def __init__(self, data: str, field_mapper: FieldMapping | None = None) -> None:
        self.data = data
        self.field_mapper = field_mapper if field_mapper is not None else FieldMapper()
        self.regex = Regex()

    def parse_string(self, key: str) -> str | None:
        identifier = self.field_mapper.field_for(key)
        return self.regex.first_match(rf"{re_escape_identifier(identifier)}(.+)\b", self.data)

    def parse_double(self, key: str) -> float | None:
        identifier = self.field_mapper.field_for(key)
        result = self.regex.first_match(rf"{re_escape_identifier(identifier)}(\w+)\b", self.data)
        return float(result) if result is not None else None

    def parse_date(self, field: str) -> datetime | None:
        date_string = self.parse_string(field)
        if not date_string or len(date_string) != 8:
            return None

        month = int(date_string[0:2])
        day = int(date_string[2:4])
        year = int(date_string[4:8])

        try:
            return datetime(year, month, day)
        except ValueError:
            return None

    def parse_first_name(self) -> str | None:
        return self.parse_string("first_name")

    def parse_last_name(self) -> str | None:
        return self.parse_string("last_name")

    def parse_middle_name(self) -> str | None:
        return self.parse_string("middle_name")

    def parse_expiration_date(self) -> datetime | None:
        return self.parse_date("expiration_date")

    def parse_is_expired(self) -> bool:
        expiration_date = self.parse_expiration_date()
        return expiration_date is not None and datetime.now() > expiration_date

    def parse_issue_date(self) -> datetime | None:
        return self.parse_date("issue_date")

    def parse_date_of_birth(self) -> datetime | None:
        return self.parse_date("date_of_birth")

    def parse_country(self) -> IssuingCountry:
        country = self.parse_string("country")
        if country == "USA":
            return IssuingCountry.UNITED_STATES
        if country == "CAN":
            return IssuingCountry.CANADA
        return IssuingCountry.UNKNOWN

    def parse_truncation_status(self, field: str) -> Truncation:
        truncation = self.parse_string(field)
        if truncation == "T":
            return Truncation.TRUNCATED
        if truncation == "N":
            return Truncation.NONE
        return Truncation.UNKNOWN

    def parse_gender(self) -> Gender:
        gender = self.parse_string("gender")
        if gender == "1":
            return Gender.MALE
        if gender == "2":
            return Gender.FEMALE
        return Gender.OTHER

    def parse_eye_color(self) -> EyeColor:
        color = self.parse_string("eye_color")
        mapping = {
            "BLK": EyeColor.BLACK,
            "BLU": EyeColor.BLUE,
            "BRO": EyeColor.BROWN,
            "GRY": EyeColor.GRAY,
            "GRN": EyeColor.GREEN,
            "HAZ": EyeColor.HAZEL,
            "MAR": EyeColor.MAROON,
            "PNK": EyeColor.PINK,
            "DIC": EyeColor.DICHROMATIC,
        }
        return mapping.get(color or "", EyeColor.UNKNOWN)

    def parse_name_suffix(self) -> NameSuffix:
        suffix = self.parse_string("suffix")
        return suffix_to_enum(suffix)

    def parse_hair_color(self) -> HairColor:
        color = self.parse_string("hair_color")
        mapping = {
            "BAL": HairColor.BALD,
            "BLK": HairColor.BLACK,
            "BLN": HairColor.BLOND,
            "BRO": HairColor.BROWN,
            "GRY": HairColor.GREY,
            "RED": HairColor.RED,
            "SDY": HairColor.SANDY,
            "WHI": HairColor.WHITE,
        }
        return mapping.get(color or "", HairColor.UNKNOWN)

    def parse_height(self) -> float | None:
        height_string = self.parse_string("height")
        height = self.parse_double("height")
        if not height_string or height is None:
            return None

        if "cm" in height_string.lower():
            return round(height * self.INCHES_PER_CENTIMETER)
        return height


def re_escape_identifier(identifier: str) -> str:
    """AAMVA identifiers are alphanumeric; escape for regex safety."""
    return re_escape(identifier)


def suffix_to_enum(suffix: str | None) -> NameSuffix:
    if not suffix:
        return NameSuffix.UNKNOWN
    table = {
        "JR": NameSuffix.JUNIOR,
        "SR": NameSuffix.SENIOR,
        "1ST": NameSuffix.FIRST,
        "I": NameSuffix.FIRST,
        "2ND": NameSuffix.SECOND,
        "II": NameSuffix.SECOND,
        "3RD": NameSuffix.THIRD,
        "III": NameSuffix.THIRD,
        "4TH": NameSuffix.FOURTH,
        "IV": NameSuffix.FOURTH,
        "5TH": NameSuffix.FIFTH,
        "V": NameSuffix.FIFTH,
        "6TH": NameSuffix.SIXTH,
        "VI": NameSuffix.SIXTH,
        "7TH": NameSuffix.SEVENTH,
        "VII": NameSuffix.SEVENTH,
        "8TH": NameSuffix.EIGHTH,
        "VIII": NameSuffix.EIGHTH,
        "9TH": NameSuffix.NINTH,
        "IX": NameSuffix.NINTH,
    }
    return table.get(suffix.upper(), NameSuffix.UNKNOWN)



from __future__ import annotations

from enum import Enum


class Gender(str, Enum):
    MALE = "Male"
    FEMALE = "Female"
    OTHER = "Other"
    UNKNOWN = "Unknown"


class EyeColor(str, Enum):
    BLACK = "Black"
    BLUE = "Blue"
    BROWN = "Brown"
    GRAY = "Gray"
    GREEN = "Green"
    HAZEL = "Hazel"
    MAROON = "Maroon"
    PINK = "Pink"
    DICHROMATIC = "Dichromatic"
    UNKNOWN = "Unknown"


class HairColor(str, Enum):
    BALD = "Bald"
    BLACK = "Black"
    BLOND = "Blond"
    BROWN = "Brown"
    GREY = "Grey"
    RED = "Red"
    SANDY = "Sandy"
    WHITE = "White"
    UNKNOWN = "Unknown"


class IssuingCountry(str, Enum):
    UNITED_STATES = "United States"
    CANADA = "Canada"
    UNKNOWN = "Unknown"


class Truncation(str, Enum):
    TRUNCATED = "Truncated"
    NONE = "None"
    UNKNOWN = "Unknown"


class NameSuffix(str, Enum):
    JUNIOR = "Junior"
    SENIOR = "Senior"
    FIRST = "First"
    SECOND = "Second"
    THIRD = "Third"
    FOURTH = "Fourth"
    FIFTH = "Fifth"
    SIXTH = "Sixth"
    SEVENTH = "Seventh"
    EIGHTH = "Eighth"
    NINTH = "Ninth"
    UNKNOWN = "Unknown"



"""``ParsedLicense`` is a type alias to ``License`` (upstream TS pattern)."""

from aamva_parser import License, ParsedLicense, parse

VALID = """
@
ANSI 636026080102DL00410288ZA03290015DLDAQD12345678
DCSPUBLIC
DACJOHN
DBB01311970
DBA01312035
DAJCA"""


def test_parse_returns_license_and_satisfies_parsed_license_alias() -> None:
    lic = parse(VALID)
    assert isinstance(lic, License)
    assert isinstance(lic, ParsedLicense)


def test_parsed_license_alias_is_license_class() -> None:
    assert ParsedLicense is License



"""Deprecated PascalCase entry points matching the upstream npm API.

Prefer :func:`aamva_parser.parse`, :func:`aamva_parser.get_version`, and
:func:`aamva_parser.is_expired` (snake_case, PEP 8).
"""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from aamva_parser.parsed_license import ParsedLicense


def Parse(barcode: str) -> ParsedLicense:
    from aamva_parser import parse as _parse

    return _parse(barcode)


def GetVersion(barcode: str) -> str | None:
    from aamva_parser import get_version as _get_version

    return _get_version(barcode)


def IsExpired(barcode: str) -> bool:
    from aamva_parser import is_expired as _is_expired

    return _is_expired(barcode)


__all__ = ["GetVersion", "IsExpired", "Parse"]



from __future__ import annotations

from aamva_parser.enums import NameSuffix
from aamva_parser.field_parser import FieldParser, suffix_to_enum
from aamva_parser.version_mappers import (
    VersionEightFieldMapper,
    VersionElevenFieldMapper,
    VersionFiveFieldMapper,
    VersionFourFieldMapper,
    VersionNineFieldMapper,
    VersionOneFieldMapper,
    VersionSevenFieldMapper,
    VersionSixFieldMapper,
    VersionTenFieldMapper,
    VersionThreeFieldMapper,
    VersionTwelveFieldMapper,
    VersionTwoFieldMapper,
)


class VersionOneFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionOneFieldMapper())

    def parse_first_name(self) -> str | None:
        first_name = self.parse_string("first_name")
        return first_name or self._parse_driver_license_name("first_name")

    def parse_last_name(self) -> str | None:
        last_name = self.parse_string("last_name")
        return last_name or self._parse_driver_license_name("last_name")

    def parse_middle_name(self) -> str | None:
        middle_name = self.parse_string("middle_name")
        return middle_name or self._parse_driver_license_name("middle_name")

    def parse_height(self) -> float | None:
        height_in_feet_and_inches = self.parse_string("height")
        if not height_in_feet_and_inches:
            return None

        height = self.regex.first_match(r"([0-9]{1})", height_in_feet_and_inches)
        inches = self.regex.first_match(r"[0-9]{1}([0-9]{2})", height_in_feet_and_inches)

        if not height or not inches:
            return None

        calculated_height = (float(height) * 12) + float(inches)

        if "cm" in height_in_feet_and_inches.lower():
            return round(calculated_height * self.INCHES_PER_CENTIMETER)
        return calculated_height

    def parse_name_suffix(self) -> NameSuffix:
        suffix = self.parse_string("suffix") or self._parse_driver_license_name("suffix")
        return suffix_to_enum(suffix)

    def _parse_driver_license_name(self, key: str) -> str | None:
        driver_license_name = self.parse_string("driver_license_name")
        if not driver_license_name:
            return None

        name_pieces = driver_license_name.split(",")
        if key == "last_name":
            return name_pieces[0].strip() if name_pieces else None
        if key == "first_name":
            return name_pieces[1].strip() if len(name_pieces) > 1 else None
        if key == "middle_name":
            return name_pieces[2].strip() if len(name_pieces) > 2 else None
        if key == "suffix":
            return name_pieces[3].strip() if len(name_pieces) > 3 else None
        return None


class VersionTwoFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionTwoFieldMapper())


class VersionThreeFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionThreeFieldMapper())


class VersionFourFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionFourFieldMapper())


class VersionFiveFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionFiveFieldMapper())


class VersionSixFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionSixFieldMapper())


class VersionSevenFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionSevenFieldMapper())


class VersionEightFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionEightFieldMapper())


class VersionNineFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionNineFieldMapper())


class VersionTenFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionTenFieldMapper())


class VersionElevenFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionElevenFieldMapper())


class VersionTwelveFieldParser(FieldParser):
    def __init__(self, data: str) -> None:
        super().__init__(data, VersionTwelveFieldMapper())
